;************************************************************************/
;*								                                        */
;*			Access Dallas 1-Wire Devices                                */
;*                                                                      */
;*          Author: Peter Dannegger                                     */
;*          modifiziert von Raik Apelt                                  */
;*                                                                      */
;************************************************************************/
;
;Voraussetzungen: ftakt - Wert fr Systemtakt mu gesetzt sein
;		          Register rmp/work/temp >=R16 mssen definiert sein
;
;-------------------------------------------------------------------------
;				Constant definitions
;-------------------------------------------------------------------------
.equ	MATCH_ROM	= 0x55
.equ	SKIP_ROM	= 0xCC
.equ	SEARCH_ROM	= 0xF0
;
.equ	CONVERT_T	= 0x44
.equ	READ_T		= 0xBE
.equ	WRITE_T		= 0x4E
.equ	STORE_EE	= 0x48
.equ	RECALL_EE	= 0xB8
;
.equ	SEARCH_FIRST	= 0xFF	; start new search
.equ	PRESENCE_ERR	= 0xFF
.equ	DATA_ERR		= 0xFF
.equ	LAST_DEVICE		= 0x00
; 0x01 ... 0x7F: continue searching
;
;device codes
.equ	Device_Code_DS1820	= 0x10
.equ	Device_Code_DS18B20	= 0x28
;
;-------------------------------------------------------------------------
;				Definitionen SRAM
;-------------------------------------------------------------------------
.equ	AddrW1		   = $100
.equ	FlagW1		   = AddrW1
.equ	W1ResetDetect  = 0
.equ	W1MsgShort     = 1
.equ	W1BusError     = 2
.equ	W1NoTempSensor = 3
.equ	W1Convert 	   = 4
.equ	TempSenVorz	   = 5			; Vorzeichen Temperaturangabe - 1 entspricht negativ
.equ	TempSenNr	   = AddrW1 + 1 ; Angabe, welcher Sensor gerade ausgelesen wird
.equ	TempSen1	   = AddrW1 + 2 ; Angabe der Temperatur in ganzen Grad
.equ	TempSen2	   = AddrW1 + 3 ; Angabe der Temperatur nach dem Komma
.equ	TempSenAddr	   = AddrW1 + 4 ; Angabe der Temperatursensoradresse
.equ	TempROM		   = AddrW1 + 5 ; Zwischenspeicher Eingangswert bei ROM-Search
.equ	W1Delay		   = AddrW1 + 6 ; Zhler fr Verzgerung von start_measure
;
;-------------------------------------------------------------------------
;				Benutzte Register
;-------------------------------------------------------------------------
.def	count		= R26
.def	sensors		= R27
.def	temp_h		= R28
.def	temp_l		= R29
;.equ	romcode		= 1				;r1..r8 store 1 wire address
.equ	romcode		= AddrW1 + 7	
;-------------------------------------------------------------------------

;-------------------------------------------------------------------------
;				Port definitions
;-------------------------------------------------------------------------
; Definition Umschalter notwendig

.equ	w1_out		= PORTD
.equ	w1_ddr		= DDRD
.equ	w1_in		= PIND
.equ	w1_pin		= PD3		;1-wire data line
;
.CSEG
;-------------------------------------------------------------------------
;				1-Wire Initialisierung
;-------------------------------------------------------------------------
;Input:  
;Output: 
;Used:	 rmp
;
w1_init:
	clr rmp
	sts FlagW1, rmp		; Flags fr W1 lschen
	sts TempSenNr, rmp
	sts TempSen1, rmp
	sts TempSen2, rmp
	sts TempSenAddr, rmp
	ldi rmp, 10
	sts W1Delay, rmp
	ldi	sensors, 0		; zu Testzwecken nur 1. Sensor abfragen
	ldi count, 0
	ret
;
;-------------------------------------------------------------------------
;				Generate 1-Wire Reset
;-------------------------------------------------------------------------
;
;                              |-Master Rx "Presence Pulse"-|
;  |---Master Reset Tx Pulse---| |----t(RSTH)---------------|
; __         t(RSTL)             _____          _____ \\ ___
;   \                           /     \        /            \
;    \_________________________/       \______/               \__ ...
;                               |-----|-------|
;                                t(PDH) t(PDL)
;                     t(R) <-- |-|
;
;	480us<=t(RSTL)< . t(RSTL)+t(R)<960us
;	480us<=t(RSTH)<
;	15us<=t(PDH)<=60us
;	60us<=t(PDL)<=240us
;
;Input:  w1_pin
;Output: W1ResetDetect = 1 im FlagW1 und C- Flag nicht gesetzt,
;		 wenn Gert erkannt wurde
;Used:	 rmp, work, YH, YL, t- Flag temporr im Statusregister
;
w1_reset:
	lds work, FlagW1
	cbr work, W1ResetDetect ; Flag W1ResetDetect im Hilfsregister zurcksetzen
	in rmp, sreg		; Statusregister holen
	push rmp		    ; Statusregister sichern wegen Benutzung T-Flag	
	ldi YH, high(480 * ftakt / 1000000 / 9) ;480s Initialisierung
	ldi YL, low(480 * ftakt / 1000000 / 9)	
_w1r1:
	cbi w1_out, w1_pin	;2 Takte - Ausgang 0 setzen, Master Reset Pulse
	sbi w1_ddr, w1_pin	;2 - low - Datenrichtung auf Ausgang setzen
	set			        ;1 T-Flag setzen
	sbiw YL, 1			;2
	brne _w1r1		    ;2

	ldi YH, high(480 * ftakt / 1000000 / 10) ; 480s Sensorreaktionen abwarten
	ldi YL, low(480 * ftakt / 1000000 / 10)	
	cli			        ;  Interrups deaktivieren
_w1r2:	
	cbi w1_ddr, w1_pin	;2 - high - Datenrichtung auf Eingang setzen
	sbic w1_in, w1_pin	;2 berspringe, wenn 0 am Eingang
	clt			        ;  Reset vom Sensor realisiert, T-Flag gelscht
	brts _w1r3		    ;2 Verzweige, wenn t-Flag gesetzt
	sbis w1_in, w1_pin	;  berspringe, wenn 1 am Eingang
	sbr work, W1ResetDetect ; Flag W1ResetDetect im Hilfsregister setzen, Sensor erkannt
_w1r3:	
	sbiw YL, 1			;2
	brne _w1r2		    ;2
	sts FlagW1, work	;  Flags fr W1 setzen

	pop rmp			    ;  Sicherkeitskopie Statusregister holen
	out sreg, rmp		;  Sicherkeitskopie Statusregister zurckschreiben
	sec					;  C-Flag setzen
	sbrc work, W1ResetDetect ; berspringen, wenn Bit W1ResetDetect nicht gesetzt ist
	clc					;  C-Flag lschen
	reti			    ;  Interrups aktivieren, Rcksprung
;
;-------------------------------------------------------------------------
;				Read / Write Bit on 1-wire
;-------------------------------------------------------------------------
;Input:	 C- Flag = bit
;Output: C- Flag = bit
;Used:   work
;
w1_bit_rd:
	sec					; C-Flag setzen
w1_bit_wr:
w1_bit_io:
	cli					; Interrups deaktivieren
	sbi w1_ddr, w1_pin	; - low - Datenrichtung auf Ausgang setzen
	ldi work, 15 * ftakt / 1000000 / 3
	brcc _w1io1
	cbi w1_ddr, w1_pin	; write 1 bit
_w1io1:
	dec work
	brne _w1io1
	sbis w1_in, w1_pin	; sample input
	clc					; C-Flag lschen
	ldi work, 45 * ftakt / 1000000 / 3
_w1io2:
	dec work
	brne _w1io2
	cbi w1_ddr, w1_pin	; high
	reti				; Interrups aktivieren, Rcksprung
;
;-------------------------------------------------------------------------
;				Read / Write byte
;-------------------------------------------------------------------------
;Input	rmp = byte
;Output rmp = byte
;Used	rmp, work - in w1_bit_io, temp
;
w1_byte_rd:
	ldi	rmp, 0xFF
w1_byte_wr:
	ldi	temp, 8
_w1wr1:
	ror	rmp			; lsb first
	cli				; because 3th stack level, Interrups deaktivieren
	rcall w1_bit_io
	dec	temp
	brne _w1wr1
	ror	rmp			; last bit in
	ret
;
;-------------------------------------------------------------------------
;				ROM Search
;-------------------------------------------------------------------------
;Input	rmp = bit to resolve
;Output	rmp = next bit to resolve
;Used	rmp, work, temp, YH, YL, zl
w1_rom_search:
	sts	TempROM, rmp
	rcall w1_reset
	ldi	rmp, presence_err
	brcs _wrs7
	ldi	rmp, search_rom
	rcall w1_byte_wr
	ldi ZH, high(romcode)
	ldi ZL, low(romcode)
	;ldi	zl, romcode		 ;point to rom code memory
	ldi	rmp, last_device
	ldi	temp, 8 * 16 - 1 ;8 bytes a 8 bit
_wrs1:
	rcall w1_bit_rd		 ;read bit
	sbc	work, work		 ;H = C
	rcall w1_bit_rd		 ;read complement bit
	ld	work, z
	brhc _wrs2
	brcc _wrs4			 ;bit = 1
	ldi	rmp, data_err
	ret
_wrs2:
	brcs _wrs5			 ;bit = 0
	lds rmp, TempROM
	cp temp, rmp
	breq _wrs5			 ;bit = 0
	brcs _wrs3			 ;bit = 1, resolve
	sbrs work, 0
	rjmp _wrs5			 ;bit = 0
_wrs3:
	mov	rmp, temp		 ;bit = 1
_wrs4:
	sec
	rjmp _wrs6
_wrs5:
	clc
_wrs6:
	ror	work
	st z, work
	rol	work
	rcall w1_bit_wr
	subi temp, 2		 ;after 8 times: H = 1
	brhc _wrs1
	adiw zh:zl, 1
	;inc	zl			 ;next byte
	brcc _wrs1
_wrs7:
	ret
;
;************************************************************************/
;*								                                     	*/
;*			Temperaturmessung			                            	*/
;*                                                                      */
;************************************************************************/
;
_temperaturmessung:
	lds rmp, FlagW1
	sbic w1_in, w1_pin			;
	rjmp _rse1
	sbr rmp, W1MsgShort 		;Flag W1MsgShort im Hilfsregister setzen
	sts FlagW1, rmp 			;Flags fr W1 setzen
	ret
_rse1:
	cbr rmp, W1MsgShort 		;Flag W1MsgShort im Hilfsregister lschen
	sts FlagW1, rmp 			;Flags fr W1 setzen
	ldi	rmp, search_first		;search sensor
	ldi	count, 0
_rse2:
	cpi	rmp, last_device
	brne _rse3
	ldi	sensors, 0
	ldi	rmp, search_first
_rse3:
	lds work, FlagW1
	sbrs work, W1Convert		;skip during conversion
	rcall w1_rom_search
	cpi	rmp, presence_err
	brne _rse4
	lds rmp, FlagW1
	sbr rmp, W1BusError 		;Flag W1BusError im Hilfsregister setzen
	sts FlagW1, rmp 			;Flags fr W1 setzen
	ret
_rse4:
	lds rmp, FlagW1
	cbr rmp, W1BusError 		;Flag W1BusError im Hilfsregister lschen
	sts FlagW1, rmp 			;Flags fr W1 setzen
	inc	count
	cp sensors, count			;selected sensor found
	brcc _rse2
;-------------------------------------------------------------------------
	sts TempSenNr, count		;display number
	lds work, FlagW1
	sbrc work, W1Convert
	ret
;-------------------------------------------------------------------------
	ldi ZH, high(romcode)
	ldi ZL, low(romcode)
	;ldi	zl, romcode
	ld	rmp, z					;device code
	cpi	rmp, Device_Code_DS1820		;DS1820
	ldi	count, 4
	breq _rdt1
	ldi	count, 1
	cpi	rmp, Device_Code_DS18B20	;DS18B20
	breq _rdt1
	lds rmp, FlagW1
	sbr rmp, W1NoTempSensor	;Flag W1NoTempSensor im Hilfsregister setzen
	sts FlagW1, rmp			;Flags fr W1 setzen
	rjmp display_address
_rdt1:
	lds rmp, FlagW1
	cbr rmp, W1NoTempSensor	;Flag W1NoTempSensor im Hilfsregister lschen
	sts FlagW1, rmp			;Flags fr W1 setzen
	ldi	rmp, READ_T			;read temperature
	rcall w1_byte_wr
	rcall w1_byte_rd		;low byte
	mov	temp_l, rmp
	rcall w1_byte_rd		;high byte
	mov	temp_h, rmp
	rjmp _rdt3
_rdt2:
	lsl	temp_l				;DS1820: 9 bit -> 12 bit
	rol	temp_h
_rdt3:
	dec	count
	brne _rdt2
;-------------------------------------------------------------------------
	lds rmp, FlagW1
	cbr rmp, TempSenVorz	;Flag Temperaturvorzeichen lschen fr positive Temperaturwerte
	sts FlagW1, rmp			;Flags fr W1 setzen
	sbrs temp_h, 7			;Test auf negative Temperaturen
	rjmp _sig1
	lds rmp, FlagW1
	sbr rmp, TempSenVorz	;Flag Temperaturvorzeichen setzen fr negative Temperaturwerte
	sts FlagW1, rmp			;Flags fr W1 setzen
	neg	temp_h				;two's complement
	neg	temp_l
	sbci temp_h, 0
_sig1:
;-------------------------------------------------------------------------
	swap temp_h
	swap temp_l
	eor	temp_h, temp_l
	andi temp_l, 0xF0	;clear bit3..0
	eor	temp_h, temp_l
	sts TempSen1, temp_h
;-------------------------------------------------------------------------
; convert 1/16 -> 1/10: n * 16 + 13 / 26 (exact: n * 16 / 25.6)
	subi	temp_l, -0x0D			;+ 0x0D
	ldi	rmp, '0' - 1
_dte1:
	inc	rmp
	subi temp_l, 0x1A			;/ 0x1A
	brcc _dte1
	sts TempSen1, rmp
;-------------------------------------------------------------------------
;				Display 1 wire address
;-------------------------------------------------------------------------
display_address:
	ldi ZH, high(romcode)
	ldi ZL, low(romcode)
	;ldi	zl, romcode
_das1:
	ld rmp, z
	sts TempSenAddr, rmp
	adiw zh:zl, 1
	;inc	zl
	lds R1, high(romcode + 8)
	lds R2, low(romcode + 8)
	;cpi	zl, romcode+8
	cp zl, R2
	cpc zh, R1
	brne _das1
	ret
;
;-------------------------------------------------------------------------
;				Start Measuring and parasite power
;-------------------------------------------------------------------------
start_measure:
	sbis w1_in, w1_pin		 	;if short circuit
	ret							;then abort
	rcall w1_reset
	ldi	rmp, SKIP_ROM
	rcall w1_byte_wr
	ldi	rmp, CONVERT_T
	rcall w1_byte_wr			;start measuring
	sbi	w1_out, w1_pin
	sbi	w1_ddr, w1_pin			;parasite power on
	ret
;-------------------------------------------------------------------------

