;-------------------------------------------------------------------------
;				Voraussetzungen
;-------------------------------------------------------------------------
; Register rmp mu definiert und temporr benutzbar sein
; Konstante ftakt fr Systemtakt mu definiert sein
;
;-------------------------------------------------------------------------
;				Port definitions
;-------------------------------------------------------------------------
; Definition Umschalter notwendig

.equ	w1_out		= PORTD
.equ	w1_ddr		= DDRD
.equ	w1_in		= PIND
.equ	w1_pin		= PD3		;1-wire data line
;
;-------------------------------------------------------------------------
;				Definitionen SRAM
;-------------------------------------------------------------------------
.equ	AddrW1		   = $100
.equ	FlagW1		   = AddrW1
.equ	W1ResetDetect  = 0
.equ	W1MsgShort     = 1
.equ	W1BusError     = 2
.equ	W1NoTempSensor = 3
.equ	W1Convert 	   = 4
.equ	TempSenVorz	   = 5			; Vorzeichen Temperaturangabe - 1 entspricht negativ
.equ	TempSenNr	   = AddrW1 + 1 ; Angabe, welcher Sensor gerade ausgelesen wird
.equ	TempSen1	   = AddrW1 + 2 ; Angabe der Temperatur in ganzen Grad
.equ	TempSen2	   = AddrW1 + 3 ; Angabe der Temperatur nach dem Komma
.equ	TempSenAddr	   = AddrW1 + 4 ; Angabe der Temperatursensoradresse
.equ	TempROM		   = AddrW1 + 5 ; Zwischenspeicher Eingangswert bei ROM-Search

.equ 	temperature_lsb	=	AddrW1 + 6		;speicherstellen definieren
.equ 	temperature_msb	=	AddrW1 + 7
.equ 	TH_register		=	AddrW1 + 8
.equ 	TL_register		=	AddrW1 + 9
.equ 	res_byte4		=	AddrW1 + 10
.equ 	res_byte5		=	AddrW1 + 11
.equ 	count_remain	=	AddrW1 + 12
.equ 	count_per_c		=	AddrW1 + 13
.equ 	crc				=	AddrW1 + 14
;
;-------------------------------------------------------------------------
;				Benutzte Register
;-------------------------------------------------------------------------
.def rBin1H = R0;	; Namensvergebung fr die verwendeten Register
.def rBin1L = R1;
.def rBin2H = R2;
.def rBin2L = R3;


.def befehl	=	R4;			;enthlt den auszugebenden 8bit-befehl
.def temp1	=	r18
.def temp2	=	r19
.def temp3	=	r26

.CSEG 

;************************************************************************/
;*								                                     	*/
;*			Temperaturmessung			                            	*/
;*                                                                      */
;************************************************************************/
;-------------------------------------------------------------------------
;				1-Wire Initialisierung
;-------------------------------------------------------------------------
;Input:  
;Output: 
;Used:	 rmp
;
w1_init:
	clr rmp
	sts FlagW1, rmp		; Flags fr W1 lschen
	sts TempSenNr, rmp
	sts TempSen1, rmp
	sts TempSen2, rmp
	sts TempSenAddr, rmp
	;ldi	sensors, 0		; zu Testzwecken nur 1. Sensor abfragen
	;ldi count, 0
	ret

_temperaturmessung:
	rcall reset_pulse
	rcall skip_rom
	rcall convert_t
	rcall reset_pulse
	rcall skip_rom
	rcall read_scratchpad
	rcall reset_pulse		;jetzt mssten die werte in den 
							;entsprechenden speicherstellen stehen
	ret

;******************************************************************************
;*****         ds18s20 routinen											  *****
;******************************************************************************

reset_pulse:
	sbi w1_ddr, w1_pin			;als ausgang
	cbi w1_out, w1_pin	
	rcall wait500us				;mcu hlt bus fr mindestens 480us low
	cbi w1_ddr, w1_pin			;lsst den bus los, geht in rx-mode
	rcall wait70us				;wartet 70 us
	
wait_presence_pulse:
	sbic w1_in, w1_pin
	rjmp wait_presence_pulse		;wartet auf presence pulse
	
wait_presence_pulse_end:
	sbis w1_in, w1_pin				
	rjmp wait_presence_pulse_end 	;wartet bis presence pulse fertig

	rcall wait410us				;wartet 412 us
	ret
;******************************************************************************

write_1:					;schreibt eine 1
	cbi w1_out, w1_pin
	sbi w1_ddr, w1_pin		;bus auf low
	rcall wait6us			;wartet 8us (maximum sind 15us)
	cbi w1_ddr, w1_pin		;lsst bus los, nun durch ext R auf high
	rcall wait64us			;slot muss mind. 60us lang sein
	ret

;******************************************************************************
write_0:				;schreibt eine 0
	cbi w1_out, w1_pin
	sbi w1_ddr, w1_pin	;bus auf low
	rcall wait60us
	cbi w1_ddr, w1_pin	;lsst bus los, nun durch ext R auf high
	rcall wait10us		;slot muss mind. 60us lang sein
	ret

;******************************************************************************
write_command:				;gibt einen befehl aus, lsb zuerst

	sbrs befehl, 0	
	rcall write_0
	sbrc befehl, 0
	rcall write_1
	
	ror befehl				;nchstes bit
	ldi temp1, 7

write_command_1:
	sbrs befehl, 0	
	rcall write_0
	sbrc befehl, 0
	rcall write_1
	ror befehl
	dec temp1
	brne write_command_1
	ret

;******************************************************************************
skip_rom:					;gibt den befehl "Skip Rom" an den
	ldi temp1, 0xCC			;sensor aus
	mov befehl, temp1
	rcall write_command
	ret

;******************************************************************************
convert_t:					;gibt den befehl "Convert T" an den
	ldi temp1, 0x44			;sensor aus
	mov befehl, temp1
	rcall write_command			

	cbi w1_out, w1_pin
	sbi w1_ddr, w1_pin		;bus low fr 6us
	rcall wait6us
	cbi w1_ddr, w1_pin		;bus loslassen
	rcall wait9us			;nach 9us samplen
convert_t_1:
	sbis w1_in, w1_pin		;wartet bis conversion zu ende ist
	rjmp convert_t_1
	ret

;******************************************************************************
read_scratchpad:				;gibt den befehl "Read Scratchpad" 
	ldi temp1, 0xBE				;an den sensor aus
	mov befehl, temp1
	rcall write_command
	rcall read_bit				;nach dem befehl werden alle 9 byte
	sts temperature_lsb, temp3	;geschickt und eingelesen
	rcall read_bit
	sts temperature_msb, temp3
	rcall read_bit
	sts TH_register, temp3
	rcall read_bit
	sts TL_register, temp3
	rcall read_bit
	sts res_byte4, temp3
	rcall read_bit
	sts res_byte5, temp3
	rcall read_bit
	sts count_remain, temp3
	rcall read_bit
	sts count_per_c, temp3
	rcall read_bit
	sts crc, temp3
	ret

;*****************************************************
read_bit:
	ldi temp3, 0			;temp3 auf 0
	ldi temp1, 8			;8mal durchfhren

read_bit_0:
	ror temp3
	cbi w1_out, w1_pin
	sbi w1_ddr, w1_pin		;bus low fr 6us
	rcall wait6us
	cbi w1_ddr, w1_pin		;bus loslassen
	rcall wait9us			;nach 9us samplen

	sbis w1_in, w1_pin		;schaut nach ob nun eine 0 anliegt
	rjmp read_bit_1			;wenn ja, springe weiter
	ldi temp2, 128
	add temp3, temp2
	rjmp read_bit_2

read_bit_1:				
	ldi temp2, 0
	add temp3, temp2

read_bit_2:
	rcall wait55us
	dec temp1
	brne read_bit_0				;zum schluss steht das fertige byte in tmp3
	ret

;******************************************************************************
wait500us:
	ldi YH, high(500 * ftakt / 1000000 / 4) ;500s
	ldi YL, low(500 * ftakt / 1000000 / 4)
wait500us_1:
	sbiw YL, 1
	brne wait500us_1
	ret
;******************************************************************************
wait410us:
	ldi YH, high(410 * ftakt / 1000000 / 4) ;480s
	ldi YL, low(410 * ftakt / 1000000 / 4)	
wait410us_1:
	sbiw YL, 1
	brne wait410us_1
	ret
;******************************************************************************
wait70us:
	ldi rmp, 70 * ftakt / 1000000 / 3
wait70us_1:
	dec rmp
	brne wait70us_1
	ret
;******************************************************************************
wait64us:
	ldi rmp, 64 * ftakt / 1000000 / 3
wait64us_1:
	dec rmp
	brne wait64us_1
	ret
;******************************************************************************
wait60us:
	ldi rmp, 60 * ftakt / 1000000 / 3
wait60us_1:
	dec rmp
	brne wait60us_1
	ret
;******************************************************************************
wait55us:
	ldi rmp, 55 * ftakt / 1000000 / 3
wait55us_1:
	dec rmp
	brne wait55us_1
	ret
;******************************************************************************
wait10us:
	ldi rmp, 10 * ftakt / 1000000 / 3
wait10us_1:
	dec rmp
	brne wait10us_1
	ret
;******************************************************************************
wait9us:
	ldi rmp, 9 * ftakt / 1000000 / 3
wait9us_1:
	dec rmp
	brne wait9us_1
	ret
;******************************************************************************
wait6us:
	ldi rmp, 6 * ftakt / 1000000 / 3
wait6us_1:
	dec rmp
	brne wait6us_1
	ret
;******************************************************************************
