;************************************************************************/
;*																		*/
;*					Aquariumsteuerung	            					*/
;*                                                                      */
;*              	Author: Raik Apelt                                  */
;*                                                                      */
;************************************************************************/
;
; Bestandteile:
; - DCF77- Decodierung
; - Realisierung einer Menanzeige
; - Wandlung des ph-Wertes und deren Anzeige als Festkommazahl
; - Vernderung des CO2-Schaltwertes des MV und deren Speicherung im EEPROM
;
; ToDo:
; - Temperaturanzeige mittels DS18S20
; - Schaltuhr
;
.NOLIST
.include "m8def.inc"
.LIST
;-------------------------------------------------------------------------
;				Register - Definitionen
;-------------------------------------------------------------------------
; Definitionen der Arbeitsregister, Register zur Berechnung (R0..R14) werden nicht definiert, da nur
; temporr mit realen Namen benutzt
; Register R26..R31 bleiben Pointerregistern vorbehalten
 
; Register R15 fr RTC benutzt
.def	rmp 		= R16	; Arbeitsregister
.def	work		= R17	; Arbeitsregister
.def	temp 		= R18	; Arbeitsregister (in 1wire benutzt)
.def	rmp2		= R19	; Arbeitsregister (in 1wire benutzt)
; Register R20...R25 fr RTC benutzt

;-------------------------------------------------------------------------
;				E/A - Definitionen
;-------------------------------------------------------------------------
.equ key_port 	 	= PinD	; Festlegung des Tasteneingangsports
.equ key_mehr		= PIND4
.equ key_weniger	= PIND5
.equ key_Menue		= PIND6
.equ key_sichern	= PIND7

.equ out_port_ddr 	= DDRC	; Datenrichtungsregister fr Ausgnge
.equ out_port 	  	= PortC	; Ausgnge
.equ Lanpe			= 4
.equ MV				= 5

;-------------------------------------------------------------------------
;				Konstanten - Definitionen
;-------------------------------------------------------------------------
; Definitionen fr LCD_Basisroutinen und DCF77- Decodierung
;
.equ pLcdPort = PortB		; Ausgangsport fr LCD-Anzeige ist PortB
.equ pLcdDdr = DDRB			; Datenrichtungsregister fr LCD-Anzeige ist DDRB
.equ ftakt = 8000000		; 8MHz - Systemtakt
;.equ ftakt = 4000000		; 4MHz - Systemtakt

;-------------------------------------------------------------------------
;				Definitionen SRAM
;-------------------------------------------------------------------------
.DSEG
PrellenOk:				.BYTE	1 	; Zwichenspeicher Tastenabfrage -> erste Speicherzelle im SRAM
WertL: 					.BYTE	1 	; Wertregister low Analogwert
WertH: 					.BYTE	1 	; Wertregister high Analogwert
Zehner:					.BYTE	1 	; Speicherzelle im SRAM: Analogwert Zehner
Einer:					.BYTE	1 	; Speicherzelle im SRAM: Analogwert Einer
Punkt:					.BYTE	1 	; Speicherzelle im SRAM: Analogwert Trennzeichen
Zehntel:				.BYTE	1 	; Speicherzelle im SRAM: Analogwert Zehntel
Hundertstel:			.BYTE	1	; Speicherzelle im SRAM: Analogwert Hundertstel
Term:					.BYTE	1 	; Speicherzelle im SRAM: Analogwert Terminierung
Menue:					.BYTE	1	; Wert, welches Men angezeigt werden soll
Menuemerker:			.BYTE	1	; Merker fr Menanzeige
SchaltwertL:			.BYTE	1 	; ph-Schaltwert low
SchaltwertH:			.BYTE	1 	; ph-Schaltwert high
ZaehlerTasteMehr:		.BYTE	1	; Zhler fr "langsame" Durchlufe Taste Mehr
ZaehlerTasteWeniger:	.BYTE	1	; Zhler fr "langsame" Durchlufe Taste Weniger
KeyStatus:				.BYTE	1	; Abbild der eingelesenen Tasten am Eingangsport
KeyMerker:				.BYTE	1	; Merker der eingelesenen Tasten am Eingangsport -> Tasten eingeschwungen?

.equ 	UeLauf			= 0x0150	; SRAM-Zelle fr Sicherheitstest / Vorwarnung SRAM- berlauf

;-------------------------------------------------------------------------
.CSEG
.ORG $000

;Interrupteinsprungadressen definieren
		rjmp INITIAL	;RESET		-	Bearbeitung
		reti			;INT0		-	externer Interrupt_0
		reti			;INT1		-	externer Interrupt_1
		reti			;TIM2_COMP	-	TIMER2 COMPARE
		rjmp TIMER_2	;TIM2_OVF	-	TIMER2 OVERVLOW
		reti			;TIM1_CAPT	-	TIMER1 CAPTURE
		reti			;TIM1_COMPA	-	TIMER1 COMPARE_A
		reti			;TIM1_COMPB	-	TIMER1 COMPARE_B
		reti			;TIM1_OVF	-	TIMER1 Overflow -> in Modul dfc77dec.asm enthalten 
		reti			;TIM0_OVF	-	TIMER0 Overflow
		reti			;SPI		-	SPI Work beendet
		reti			;UART_RX	-	RX_Byte empfangen
		reti			;UART_DRE	-	UDRE_Datenregister leer
		reti			;UART_TX	-	TX_Work beendet
		reti			;ADC		-	analog-digital Wandlung
		reti			;EE_RDY		-	EEPROM ready
		reti			;ANA_COMP	-	Analog Komparator
		reti			;TWSI		-	Two-Wire Serial Interface
		reti			;SPM_RDY	-	Store Program Memory Ready

; Interruptroutinen
;TIMER_0:	;push rmp		; Arbeitsregister sichern
			;in	rmp, sreg	; Statusregister holen
			;push rmp		; Statusregister sichern
						
			;pop rmp		; Sicherkeitskopie Statusregister holen
			;out sreg, rmp	; Sicherkeitskopie Statusregister zurckschreiben
			;pop rmp		; Arbeitsregister zurckschreiben
			;reti			; zurck aus Interrupt und erneute Freigabe Interrupts

; AVR Initialisierung
INITIAL:
			ldi rmp, high(RAMEND)
			out SPH, rmp
			ldi rmp, low(RAMEND)
			out	SPL, rmp			; Stackpointer initialisieren

			rcall Lcd4Init     		; Display initialisieren und Anzeige lschen, Festlegung des Displayports

; Register und Speicherzellen initialisieren
			ldi rmp, 0x00
			sts PrellenOk, rmp		; Startwert fr Testenprellmerker
						
			clr rmp
			sts Menue, rmp			; Menwahl Vorgabe laden

			sts UeLauf, rmp			; Sicherheitszelle mit Wert laden zum Test von berlauf SRAM

			ldi rmp, 0xF5			; Startwerte fr Tastenzhler langsame Durchlufe in SRAM laden
			sts ZaehlerTasteMehr, rmp
			sts ZaehlerTasteWeniger, rmp
			
			clr rmp					; Menmerker initialisieren -> EEPROM-Werte nur einmalig lesen
			sts Menuemerker, rmp

; Funkuhrmodul dcf77 initialisieren
			rcall _dcf77init

; Ports initialisieren
			ldi rmp, 1<<key_mehr^1<<key_weniger^1<<key_Menue^1<<key_sichern
			out	key_port, rmp		; in PortD wird teilweise einen PullUp-Widerstand eingeschalten, der bei
									; offenem Eingang undefinierte Zustnde vermeidet
			
			ldi rmp, 0b11110000       	
        	out out_port_ddr, rmp   ; Pin vom E/A-Port sind jetzt Ausgnge durch Ausgabe 
									; in Datenrichtungs-Register auer Pin0

			cbi out_port, MV 		; Bit5 (MV) in out_port auf 0 setzen
			
; A/D-Wandler initialisieren
			clr rmp
			out ADMUX,rmp		;Pin0 von PortC als Analogeingang im MUX setzen
			ldi rmp,0b11100111
			out ADCSR,rmp		;A/D-Wandler ein, Freemode, Interrupt nicht aktiv, Teiler=128

; Intro LCD abspielen
			rcall intro

; Temperatursensoren initialisieren
			rcall w1_init

; Timer initialisieren fr dcf77-Modul und Tastenentprellung
			ldi	rmp, 1<<CS22^1<<CS21 ; Teiler im Zhler 1/256
			out	TCCR2, rmp			 ; Teiler in Zhler schreiben und Timer starten
			ldi	rmp, 1<<TOIE2
			out	TIMSK, rmp			 ; TC2 overflow Interrupt freigeben 
									 ; -> beim AT4433 ist TC0 Bit1

; Ende der Initialisierung
			sei						 ; globale Interruptfreigabe
			rjmp HAUPT			

; Unterprogramme
weniger:	lds	R5, ZaehlerTasteWeniger ; Zhler fr "langsame" Durchlufe laden
			lds R1, SchaltwertL
			lds R2, SchaltwertH	; Schaltwert aus SRAM laden
			ldi rmp, 0x01
			mov R3, rmp
			ldi rmp, 0x00
			mov R4, rmp			; Subtrahenden 16bit fr Schaltwert laden
        	sub R1, R3
        	sbc R2, R4			; Schaltwert 16bit verringern
           	brmi weniger1				; zu weniger1: springen, wenn Ergebnis negativ
										; -> nicht speichern 
			sts SchaltwertH, R2		
			sts SchaltwertL, R1			; Schaltwert speichern
weniger1:	inc R5
			breq weniger2				; grere Wartezeit fr Durchlauf berspringen
			sts ZaehlerTasteWeniger, R5	; Zhler fr "langsame" Durchlufe speichern
			rcall Delay400ms
			ret
weniger2:	rcall Delay50ms
			ret

mehr:		lds R3, ZaehlerTasteMehr	; Zhler fr "langsame" Durchlufe laden
			lds R1, SchaltwertL
			lds R2, SchaltwertH	; aktuellen Schaltwert laden
			inc R1				; SchaltwertL um 1 erhhen
			brne mehr1			; wenn kein berlauf stattfndet, dann zu mehr1:
			inc R2				; SchaltwertH um 1 erhhen
			ldi rmp, 0xFC	
			add rmp, R2			; high Endwert fr A/D- Wandler mit 0xFC zwecks berlauftest
								; addieren -> berlauf dann bei 0x03FF des 16bit Schaltwertes
			brcs mehr2			; wenn c-Flag (berlauf) gesetzt wurde, dann springe zu mehr2: -> nicht speichern
mehr1:		sts SchaltwertH, R2		
			sts SchaltwertL, R1	; Schaltwert speichern
mehr2:		inc R3
			breq mehr3
			sts ZaehlerTasteMehr, R3	; Zhler fr "langsame" Durchlufe speichern
			rcall Delay400ms
			ret
mehr3:		rcall Delay50ms
			ret

Menuewahl:	lds rmp, Menue		; alte Menauswahl aus SRAM laden
			dec rmp				; Menwahl um 1 mindern
			brmi Menuewahl1		; Verzweige wenn negativer Wert
			sts Menue, rmp		; neue Menauswahl im SRAM abspeichern
			rcall Delay400ms	; nach Tastendruck 400ms warten
			rcall Lcd4Clear		; LCD-Anzeige lschen
			ret
Menuewahl1:
			ldi rmp, 4			; Platz fr 5 Menpunkte
			sts Menue, rmp		; neue Menauswahl im SRAM abspeichern
			rcall Delay400ms	; nach Tastendruck 400ms warten
			rcall Lcd4Clear		; LCD-Anzeige lschen
			ret

Wandlung:	rcall fpconv10		; Wert dezimal im SRAM ablegen
			sts Zehner, R5
			sts Einer, R6
			sts Punkt, R7
			sts Zehntel, R8
			sts Hundertstel, R9
			sts	Term, R10
			ret

; Verzgerung um 400 ms fr den Tastendruck -> bentigt LCD_Basisroutine
Delay400ms:
			ldi rmp,80			; 80 * 5 ms warten
Delay400ms1:
			push rmp 			; Rette Byte auf Stapel
			rcall LcdDelay5ms
			pop rmp 			; Stelle Byte wieder her
			dec rmp
			brne Delay400ms1
			ret

; Verzgerung um 50 ms fr den Tastendruck (Schnelldurchlauf) -> bentigt LCD_Basisroutine
Delay50ms:
			ldi rmp,10			; 10 * 5 ms warten
Delay50ms1:
			push rmp 			; Rette Byte auf Stapel
			rcall LcdDelay5ms
			pop rmp 			; Stelle Byte wieder her
			dec rmp
			brne Delay50ms1
			ret

; Routine zum Lesen von 16-bit EE-Prom Werten
; Zeiger Z - bergabewert der EEPROM-Adresse, R1 und R2 beinhalten die Werte des EEPROMs
EEPROM_Read:
		sbic EECR, EEWE			
		rjmp EEPROM_Read		; Warteschleife, bis EEPROM bereit zum Lesen
        out EEARH, ZH
		out EEARL, ZL			; EEPROM-Adresse
		sbi EECR, EERE          ; Lesevorgang starten
        in rmp, EEDR            ; gelesenes Byte nach "rmp" schreiben
        mov R2, rmp             ; Inhalt rmp an R2 bergeben (Highbyte Daten)
        adiw ZL, 1				; gesamten Zeiger Z um 1 erhhen
EEPROM_Read1:
		sbic EECR, EEWE			
		rjmp EEPROM_Read1		; Warteschleife, bis EEPROM bereit zum Lesen
		out EEARH, ZH
		out EEARL, ZL			; EEPROM-Adresse
		sbi EECR, EERE          ; Lesevorgang starten
        in rmp, EEDR            ; gelesenes Byte nach "rmp"
        mov R1, rmp             ; Inhalt rmp an R1 bergeben (Lowbyte Daten)
        ret

; Routine zum Schreiben von 16-bit EE-Prom Werten
; Zeiger Z - bergabewert der EEPROM-Adresse, R1 und R2 beinhalten die Werte fr den EEPROM
EEPROM_Write:
		sbic EECR, EEWE			
		rjmp EEPROM_Write		; Warteschleife, bis EEPROM bereit zum Schreiben
        out EEARH, ZH
		out EEARL, ZL			; EEPROM-Adresse
		mov rmp, R2             ; Inhalt R2 an rmp bergeben (Highbyte Daten)
		out EEDR, rmp           ; Byte von "rmp" nach Datenregister EEPROM schreiben
		sbi EECR, EEMWE         ; Schreibvorgang vorbereiten
		sbi EECR, EEWE			; Schreibvorgang starten
        adiw ZL, 1				; gesamten Zeiger Z um 1 erhhen
EEPROM_Write1:
		sbic EECR, EEWE			
		rjmp EEPROM_Write1		; Warteschleife, bis EEPROM bereit zum Lesen
		out EEARH, ZH
		out EEARL, ZL			; EEPROM-Adresse
		mov rmp, R1             ; Inhalt R1 an rmp bergeben (Lowbyte Daten)
		out EEDR, rmp           ; Byte von "rmp" nach Datenregister EEPROM schreiben
		sbi EECR, EEMWE         ; Schreibvorgang vorbereiten
		sbi EECR, EEWE			; Schreibvorgang starten
        ret

; Routine zum Entprellen der Tasteneingnge durch Einfgen von einer Warteschleife
ENTPRELLEN:	lds rmp, KeyStatus
			sts KeyMerker, rmp		; Kopieren Status Tasten
			rcall LcdDelay50us		; 50s warten, bentigt LCD_Basisroutine
			rjmp _Haupt3

; Hauptprogramm		
HAUPT:		in rmp,ADCL
			sts WertL, rmp
			in rmp,ADCH
			sts WertH, rmp		; aktuellen ph-Wert im SRAM ablegen

			rcall decodedcf77

			rcall _temperaturmessung ; Temperaturmessung

			brts _Haupt1
			sbrs flags, second1
			rjmp _Haupt2
			rcall clock_inc
			rcall RtcConv		; Umwandlung der Uhrzeit/Datum von BCD-RTC zu ASCII-Uhrzeit/Datum

			lds rmp, FlagW1
			cbr	rmp, 1<<W1Convert
			sts FlagW1, rmp
			lds rmp, W1Delay
			dec rmp
			sts W1Delay, rmp
			brne _Haupt1
			lds rmp, FlagW1
			sbr	rmp, 1<<W1Convert
			sts FlagW1, rmp
			rcall start_measure ; Start der Temperaturmessung am W1-Bus
			ldi rmp, 10
			sts W1Delay, rmp	; alle 10s start_measure aufrufen

_Haupt1:	cbr flags, 1<<second1
_Haupt2:
			rcall Regelung		; Zweipunktregelung

_Haupt3:	in rmp, key_port	; Einlesen der Tasten in Register
			ori rmp, 0b00001111	; Eingang key_port Bit 0,1,2,3 unbercksichtigt lassen
			sts KeyStatus, rmp
			lds work, KeyMerker
			cp work, rmp		; Vergleichen ob Tasten vom Port eingeschwungen sind
			brne ENTPRELLEN		; Sprung zum entprellen, wenn Port nicht eingeschwungen

			sbis key_port, key_mehr		; berspringe nchste Zeile, wenn Bit key_mehr gesetzt ist, da low aktiv
			rcall mehr

			ldi rmp, 0xF5				; Startwerte fr Tastenzhler in SRAM laden
			sbic key_port, key_mehr		; Taste Mehr nicht bettigt, dann...
			sts ZaehlerTasteMehr, rmp	; Zhler fr Schnelldurchlauf zurcksetzen

			sbis key_port, key_weniger	; berspringe nchste Zeile, wenn Bit key_weniger gesetzt ist, da low aktiv
			rcall weniger

			ldi rmp, 0xF5					; Startwerte fr Tastenzhler in SRAM laden
			sbic key_port, key_weniger		; Taste Weniger nicht bettigt, dann...
			sts ZaehlerTasteWeniger, rmp	; Zhler fr Schnelldurchlauf zurcksetzen

			sbis key_port, key_Menue	; berspringe nchste Zeile, wenn Bit key_Menue gesetzt ist, da low aktiv
			rcall Menuewahl
			
			lds rmp, Menue		; Menauswahl mittels Z-Register
			ldi ZH, high(Menuetabelle)
			ldi ZL, low(Menuetabelle)
			add ZL, rmp
			brcc Ueberlauf
			inc ZH
Ueberlauf:	ijmp				; Indirekter Sprung auf Marke vom Z-Pointer

Menuetabelle:
			rjmp Uhr			; Sprung zum 1. Menpunkt Anzeige Uhrzeit
			rjmp SchaltAusMV	; Sprung zum 5. Menpunkt Schaltpunkt Aus Magnetventil
			rjmp SchaltAnMV		; Sprung zum 4. Menpunkt Schaltpunkt An Magnetventil
			rjmp phAnzeige		; Sprung zum 3. Menpunkt Anzeige aktueller ph-Wert
			rjmp Temperatur		; Sprung zum 2. Menpunkt Temperaturanzeige
			
; Menpunkt zur Anzeige der aktuellen Uhrzeit und Datum auf LCD
Uhr:		ldi rmp,0x84 		
			rcall Lcd4Ctrl		; Curser verschieben in 1.Zeile 5.Zeichen in LCD
			ldi ZH, high(StuH)
			ldi ZL, low(StuH)
			ldi rmp, 8
			rcall Lcd4RTxt		; 8 Zeichen ab Z (StuH) auf LCD ausgeben
			ldi rmp,0xC4 		
			rcall Lcd4Ctrl		; Curser verschieben in 2.Zeile 5.Zeichen in LCD
			ldi ZH, high(TagH)
			ldi ZL, low(TagH)
			ldi rmp, 6
			rcall Lcd4RTxt		; 6 Zeichen ab Z (TagH) auf LCD ausgeben
			ldi rmp, '2'
			rcall Lcd4Chr
			ldi rmp, '0'
			rcall Lcd4Chr
			ldi ZH, high(JhrH)
			ldi ZL, low(JhrH)
			ldi rmp, 2
			rcall Lcd4RTxt		; 2 Zeichen ab Z (JhrH) auf LCD ausgeben
; Anzeige DCF77 Status
			ldi rmp,0x8E 		
			rcall Lcd4Ctrl		; Curser verschieben in 1.Zeile 15.Zeichen in LCD
			ldi rmp, ' '
			sbrc flags, dcf77error
			ldi rmp, 'E'
			rcall Lcd4Chr		; Fehleranzeige
			ldi rmp, ' '
			sbrs newsecond, 0
			ldi rmp, '*'
			rcall Lcd4Chr
; Test auf berlauf SRAM
			ldi rmp,0x80 		
			rcall Lcd4Ctrl
			lds rmp, UeLauf
			cpi rmp, 0
			breq _Ueberlauf
			ldi rmp, '!'
			rcall Lcd4Chr
_Ueberlauf:			
; Anzeige Wochentag
			ldi rmp,0xC0 		
			rcall Lcd4Ctrl		; Curser verschieben in 2.Zeile 1.Zeichen in LCD
			lds R1, WTAG		; Fehlerabfrage Wochentag
			ldi rmp, 7
			cp rmp, R1
			brcs ErrorWTag
			mov rmp, R1
			tst rmp
			breq ErrorWTag
; Bestimmung des Wochentages
			cpi rmp, 1
			breq MoWTag
			cpi rmp, 2
			breq DiWTag
			cpi rmp, 3
			breq MiWTag
			cpi rmp, 4
			breq DoWTag
			cpi rmp, 5
			breq FrWTag
			cpi rmp, 6
			breq SaWTag
			cpi rmp, 7
			breq SoWTag
; Wochentagsanzeigen
MoWTag:		ldi rmp, 'M'
			rcall Lcd4Chr
			ldi rmp, 'o'
			rcall Lcd4Chr
			rjmp HAUPT
DiWTag:		ldi rmp, 'D'
			rcall Lcd4Chr
			ldi rmp, 'i'
			rcall Lcd4Chr
			rjmp HAUPT
MiWTag:		ldi rmp, 'M'
			rcall Lcd4Chr
			ldi rmp, 'i'
			rcall Lcd4Chr
			rjmp HAUPT
DoWTag:		ldi rmp, 'D'
			rcall Lcd4Chr
			ldi rmp, 'o'
			rcall Lcd4Chr
			rjmp HAUPT
FrWTag:		ldi rmp, 'F'
			rcall Lcd4Chr
			ldi rmp, 'r'
			rcall Lcd4Chr
			rjmp HAUPT
SaWTag:		ldi rmp, 'S'
			rcall Lcd4Chr
			ldi rmp, 'a'
			rcall Lcd4Chr
			rjmp HAUPT
SoWTag:		ldi rmp, 'S'
			rcall Lcd4Chr
			ldi rmp, 'o'
			rcall Lcd4Chr
			rjmp HAUPT
ErrorWTag:	ldi rmp, 'E'
			rcall Lcd4Chr
			ldi rmp, 'E'
			rcall Lcd4Chr
			rjmp HAUPT

; Menpunkt der Anzeige der aktuellen Temperatur
Temperatur:	ldi rmp,0x80 		
			rcall Lcd4Ctrl		; Curser verschieben in 1.Zeile 1.Zeichen in LCD
			ldi ZH, high(_text8)
			ldi ZL, low(_text8)	; Adresse des text8 von EEPROM laden
			rcall LcdEepPrint	; Unterfunktion LcdEepPrint aufrufen
			lds R1, TempSenNr
			;mov R1, sensors		; Vorbereitung Anzeige Sensornummer 1...20
			inc R1				; Beginn Sensornummer mit 1 und nicht mit 0
			rcall BcdConv		; Umbandlung in Dezimalzahl - bentigt dcf77dec.asm
			mov rmp, R3
			rcall Lcd4Chr		; Anzeige Zehner der Sensornummer
			mov rmp, R4
			rcall Lcd4Chr		; Anzeige Einer der Sensornummer
			ldi rmp,0xC4 		
			rcall Lcd4Ctrl		; Curser verschieben in 2.Zeile 5.Zeichen in LCD
			ldi rmp, ' '
			lds work, FlagW1
			sbrc work, TempSenVorz
			ldi rmp, '-'
			rcall Lcd4Chr		; negative Temperaturwerte
			lds rmp, TempSen1
			rcall Lcd4PBcd
			ldi rmp, ','
			rcall Lcd4Chr
			lds rmp, TempSen2
			rcall Lcd4PBcd
			ldi rmp, $DF		; Zeichen "  " im LCD laden
			rcall Lcd4Chr
			ldi rmp, 'C'
			rcall Lcd4Chr
; Anzeige Temperatur- Sensor- Status
			ldi rmp,0x8D 		
			rcall Lcd4Ctrl		; Curser verschieben in LCD
			ldi rmp, ' '
			lds work, FlagW1
			sbrc work, W1MsgShort
			ldi rmp, '!'
			rcall Lcd4Chr		; Bus short
			ldi rmp, ' '
			lds work, FlagW1
			sbrc work, W1BusError
			ldi rmp, 'E'
			rcall Lcd4Chr		; Busfehler
			ldi rmp, ' '
			lds work, FlagW1
			sbrc work, W1NoTempSensor
			ldi rmp, '?'
			rcall Lcd4Chr		; kein Sensor gefunden
			sbic key_port, key_sichern	; berspringe nchste Zeile, wenn Bit key_sichern gelscht ist, da low aktiv
			rjmp Haupt
			inc	sensors				;next sensor
			rcall Delay50ms
			cpi	sensors, 20
			brcs _TempRet
			ldi	sensors, 0
_TempRet:	rjmp HAUPT
; Menpunkt der Anzeige des aktuellen ph-Wertes
phAnzeige:	lds R1, WertL
			lds R2, WertH
			rcall Wandlung
			ldi rmp,0x83 		
			rcall Lcd4Ctrl		; Curser verschieben in 1.Zeile 4.Zeichen in LCD
        	ldi ZH, high(_text1)
			ldi ZL, low(_text1)	; Adresse des text1 von EEPROM laden
        	rcall LcdEepPrint	; Unterfunktion LcdEepPrint aufrufen
			ldi rmp,0xC0 		
			rcall Lcd4Ctrl		; Curser verschieben in 2.Zeile 1.Zeichen in LCD
			ldi ZH, high(_text7)
			ldi ZL, low(_text7)	; Adresse des text7 von EEPROM laden
        	rcall LcdEepPrint	; Unterfunktion LcdEepPrint aufrufen
			ldi rmp,0xC9
			rcall Lcd4Ctrl		; Curser verschieben in 2.Zeile 10.Zeichen in LCD
			ldi ZH, high(Zehner)
			ldi ZL, low(Zehner)	; Zeiger auf Adresse fr gewandelte Zahl -> Zehnerstelle setzen
								; Unterschlagung der Zehnerstelle mglich

			ldi rmp, 5			; bergabewert fr die Anzeige der 5 nchst folgenden Zeichen
								; aus SRAM
			rcall Lcd4RTxt
			rjmp HAUPT

SchaltAnMV:
			lds rmp, Menuemerker; letzten Menmerker laden, damit nur 1mal der Schaltwert fr
			sbrc rmp, 0			; Magnetventil An aus dem EEPROM in SRAM geladen werden
			rjmp SchaltAnMV1
			ldi rmp, 0b00000001
			sts Menuemerker, rmp; nochmaliges Laden aus EEPROM sperren
			ldi ZH, high(_WertMV_An)
			ldi ZL, low(_WertMV_An)	; Adresse des Schaltwertes MV An von EEPROM laden
			rcall EEPROM_Read	; aus EEPROM Schaltwert lesen
			sts SchaltwertH, R2
			sts SchaltwertL, R1	; ausgelesenen Schaltwert speichern
SchaltAnMV1:
			lds R1, SchaltwertL	; Schaltwert aktuell aus SRAM laden zur Vorbereitung anschlieender
			lds R2, SchaltwertH	; Wandlung
			rcall Wandlung		; Schaltwert in Fliekommazahl wandeln
			ldi rmp,0x81 		
			rcall Lcd4Ctrl		; Curser verschieben in 1.Zeile 2.Zeichen in LCD
			ldi ZH, high(_text2)
			ldi ZL, low(_text2)	; Adresse des text2 von EEPROM laden
			rcall LcdEepPrint	; Unterfunktion LcdEepPrint aufrufen
			ldi rmp,0xC0 		
			rcall Lcd4Ctrl		; Curser verschieben in 2.Zeile 1.Zeichen in LCD
			ldi ZH, high(_text3)
			ldi ZL, low(_text3)	; Adresse des text3 von EEPROM laden
			rcall LcdEepPrint	; Unterfunktion LcdEepPrint aufrufen
			ldi rmp,0xC9
			rcall Lcd4Ctrl		; Curser verschieben in 2.Zeile 10.Zeichen in LCD
			ldi ZH, high(Zehner)
			ldi ZL, low(Zehner)	; Zeiger auf Adresse fr gewandelte Zahl -> Zehnerstelle setzen
								; Unterschlagung der Zehnerstelle mglich
			ldi rmp, 5			; bergabewert fr die Anzeige der 5 nchst folgenden Zeichen
								; aus SRAM
			rcall Lcd4RTxt
			sbis key_port, key_sichern	; berspringe nchste Zeile, wenn Bit key_sichern gesetzt ist, da low aktiv
			rcall SchaltAnMVstore
			rjmp HAUPT	
SchaltAnMVstore:
			ldi ZH, high(_WertMV_An)
			ldi ZL, low(_WertMV_An) ; bergabeparameter fr EEPROM Speicherung laden
			lds R1, SchaltwertL
			lds R2, SchaltwertH
			rcall EEPROM_Write	; Schaltwert Magnetventil An in EEPROM speichern
			rcall Delay400ms	; nach Tastendruck 400ms warten
			ret

SchaltAusMV:
			lds rmp, Menuemerker; letzten Menmerker laden, damit nur 1mal der Schaltwert fr
			sbrc rmp, 1			; Magnetventil Aus aus dem EEPROM in SRAM geladen werden
			rjmp SchaltAusMV1
			ldi rmp, 0b00000010
			sts Menuemerker, rmp; nochmaliges Laden aus EEPROM sperren
			ldi ZH, high(_WertMV_Aus)
			ldi ZL, low(_WertMV_Aus)
			rcall EEPROM_Read	; aus EEPROM Schaltwert lesen
			sts SchaltwertH, R2
			sts SchaltwertL, R1	; ausgelesenen Schaltwert speichern
SchaltAusMV1:
			lds R1, SchaltwertL	; Schaltwert aktuell aus SRAM laden zur Vorbereitung anschlieender
			lds R2, SchaltwertH	; Wandlung
			rcall Wandlung		; Schaltwert in Fliekommazahl wandeln
			ldi rmp,0x81 		
			rcall Lcd4Ctrl		; Curser verschieben in 1.Zeile 2.Zeichen in LCD
			ldi ZH, high(_text2)
			ldi ZL, low(_text2)	; Adresse des text2 von EEPROM laden
			rcall LcdEepPrint	; Unterfunktion LcdEepPrint aufrufen
			ldi rmp,0xC0 		
			rcall Lcd4Ctrl		; Curser verschieben in 2.Zeile 1.Zeichen in LCD
			ldi ZH, high(_text4)
			ldi ZL, low(_text4)	; Adresse des text4 von EEPROM laden
			rcall LcdEepPrint	; Unterfunktion LcdEepPrint aufrufen
			ldi rmp,0xC9
			rcall Lcd4Ctrl		; Curser verschieben in 2.Zeile 10.Zeichen in LCD
			ldi ZH, high(Zehner)
			ldi ZL, low(Zehner)	; Zeiger auf Adresse fr gewandelte Zahl -> Zehnerstelle setzen
								; Unterschlagung der Zehnerstelle mglich

			ldi rmp, 5			; bergabewert fr die Anzeige der 5 nchst folgenden Zeichen
								; aus SRAM
			rcall Lcd4RTxt
			sbis key_port, key_sichern	; berspringe nchste Zeile, wenn Bit key_sichern gesetzt ist, da low aktiv
			rcall SchaltAusMVstore
			rjmp HAUPT	
SchaltAusMVstore:
			ldi ZH, high(_WertMV_Aus)
			ldi ZL, low(_WertMV_Aus)	; bergabeparameter fr EEPROM Speicherung laden
			lds R1, SchaltwertL
			lds R2, SchaltwertH
			rcall EEPROM_Write	; Schaltwert Magnetventil Aus in EEPROM speichern
			rcall Delay400ms	; nach Tastendruck 400ms warten
			ret

.include "LCD_Basisroutine.asm"
.include "LCD-Intro.asm"
.include "Fliesskomma_10bit_ph-Wert.asm"
.include "dcf77dec.asm"
.include "CO2-Steuerung.asm"
.include "1WIRE.inc"

.eseg                   ; EEPROM-Segment aktivieren
_text1:
.db "ph-Wert", 0		; Text 1 im EEPROM terminiert mit 0
_text2:
.db "ph-Schaltwert", 0	; Text 2 im EEPROM terminiert mit 0
_text3:
.db "MV An", 0			; Text 3 im EEPROM terminiert mit 0
_text4:
.db "MV Aus", 0			; Text 4 im EEPROM terminiert mit 0
_text7:	
.db "aktuell", 0		; Text 7 im EEPROM terminiert mit 0
_text8:	
.db "Temperatur ", 0		; Text 8 im EEPROM terminiert mit 0

; 16-bit Festwerte (Highbyte, Lowbyte, Terminierung), Terminierung zur Sicherheit
_WertMV_An:
.db 0x01, 0x19, 0		; Startwert (Erstbetrieb) fr Schaltschwelle Magnetventil An
_WertMV_Aus:
.db 0x01, 0x09, 0		; Startwert (Erstbetrieb) fr Schaltschwelle Magnetventil Aus

