; Die Aufgabe: Ein 10-Bit-Analog-Wandler-Signal
; in Binrform wird eingelesen, die Zahlen
; reichen von hex 0000 bis 03FF.
; Diese Zahlen sind umzurechnen in eine
; Fliekommazahl zwischen 00,00 bis 25,00
;
; von Gerhard Schmidt
;
; Aufruf der Umwandlungsroutine:	rcall fpconv10
; Voraussetzungen
;	- Stack mu initianisiert sein
;	- Register rmp mu definirt sein ( Register >=R16 )
;	- Register R1 ... R10 drfen zum Zeitpunkt nicht in Verwendung sein
;
.CSEG
fpconv10:
	rcall fpconv10c ; Prfe die Eingabezahl in R2:R1
	brcs fpconv10e 	; Wenn Carry, dann Ergebnis="E.EEE"
    rcall fpconv10m ; Multipliziere mit 320.313
	rcall fpconv10r ; Ergebnis durch 2 teilen, Runden und Division mit 65536
	rcall fpconv10a ; Umwandlung in ASCII-String
	rjmp fpconv10f 	; Setze Dezimalpunkt und Nullabschluss
fpconv10e:
	ldi rmp,'E' ; Fehlermeldung in Ergebnistext
	mov R5,rmp
	mov R7,rmp
	mov R8,rmp
	mov R9, rmp
fpconv10f:
	ldi rmp,'.' ; Setze Dezimalpunkt
	mov R7,rmp
	clr rmp 	; Null-Abschluss setzen
	mov R10,rmp
	ret
;
; Unterprogramm Eingabeprfung
;
fpconv10c:
	ldi rmp,$03 ; Vergleiche MSB mit 03
	cp rmp,R2 	; wenn R2>$03, setze carry bei Rckkehr
	ret
;
; Unterprogramm Multiplikation mit 320.313
;
; Startbedingung:
; +---+---+
; | R2+ R1|  Eingabezahl
; +---+---+
; +---+---+---+---+
; | R6| R5| R4| R3| Multiplikant 320.313 = $00 04 E3 38
; | 00| 04| E3| 38|
; +---+---+---+---+
; +---+---+---+---+
; |R10| R9| R8| R7| Resultat
; | 00| 00| 00| 00|
; +---+---+---+---+
;
fpconv10m:
	clr R6 		; Setze den Multiplikant auf 320.313
	ldi rmp,$04
	mov R5,rmp
	ldi rmp,$E3
	mov R4,rmp
	ldi rmp,$38
	mov R3,rmp
	clr R10 	; leere Ergebnisregister
	clr R9
	clr R8
	clr R7
fpconv10m1:
	mov rmp,R1 		; Prfe ob noch Bits zu multiplizieren
	or rmp,R2 		; Irgendein Bit Eins?
	brne fpconv10m2 ; Noch Einsen, mach weiter
	ret
fpconv10m2:
	lsr R2 		; Schiebe MSB nach rechts (teilen durch 2)
	ror R1 		; Rotiere LSB rechts und setze Bit 7
	brcc fpconv10m3 ; Wenn das niedrigste Bit eine 0 war,
;		dann berspringe den Additionsschritt
    add R7,R3 ; Addiere die Zahl in R6:R5:R4:R3 zum Ergebnis
	adc R8,R4
	adc R9,R5
	adc R10,R6
fpconv10m3:
	lsl R3 ; Multipliziere R6:R5:R4:R3 mit 2
	rol R4
	rol R5
	rol R6
	rjmp fpconv10m1 ; Wiederhole fr das nchste Bit
;
; Runde die Zahl in R10:R9 mit dem Wert von Bit 7 von R8
;
fpconv10r:
	lsr R10
	ror R9
	ror R8 		; Ergebnis durch 2 teilen
	clr rmp 	; Null nach rmp
	lsl R8 		; Rotiere Bit 7 ins Carry
	adc R9,rmp 	; Addiere LSB mit bertrag
	adc R10,rmp ; Addiere MSB mit bertrag	(runden R9, R10 mit Carry- Flag von R8)
	mov R2,R10 	; Kopiere den Wert nach R2:R1 (durch 65536 teilen)
	mov R1,R9
	ret
;
; Wandle das Wort in R2:R1 in einen ASCII-Text in R5:R6:R7:R8:R9:R10
;
; +---+---+
; + R2| R1| Eingangswert 0..25.00
; +---+---+
; +---+---+
; | R4| R3| Dezimalteiler
; +---+---+
; +---+---+---+---+---+---+
; | R5| R6| R7| R8| R9|R10| Ergebnistext (fr Eingangswert 25,00)
; |'2'|'5'|'.'|'0'|'0'|$00| mit Null-Abschluss
; +---+---+---+---+---+---+
;
fpconv10a:
	ldi rmp,HIGH(1000) ; Setze Dezimalteiler auf 1.000
	mov R4,rmp
	ldi rmp,LOW(1000)
	mov R3,rmp
	rcall fpconv10d ; Hole ASCII-Ziffer durch wiederholtes Abziehen
	mov R5,rmp 	; Setze Tausender Zeichen
	clr R4 		; Setze Dezimalteiler auf 100
	ldi rmp,100
	mov R3,rmp
	rcall fpconv10d ; Hole die nchste Ziffer
	mov R6,rmp 	; Setze Hunderter Zeichen
	ldi rmp,10 	; Setze Dezimalteiler auf 10
	mov R3,rmp
	rcall fpconv10d ; Hole die nchste Ziffer
	mov R8,rmp 	; Setze Zehner Zeichen
	ldi rmp,'0' ; Wandle Rest in ASCII-Ziffer um
	add rmp,R1
	mov R9,rmp 	; Setze Einer Zeichen
	ret
;
; Wandle Binrwort in R2:R1 in eine Dezimalziffer durch fortgesetztes
; Abziehen des Dezimalteilers in R4:R3 (1000, 100, 10)
;
fpconv10d:
	ldi rmp,'0' ; Beginne mit ASCII-0
fpconv10d1:
	cp R1,R3 	; Vergleiche Wort mit Teiler
	cpc R2,R4
	brcc fpconv10d2 ; Carry nicht gesetzt, subtrahiere Teiler
	ret
fpconv10d2:
	sub R1,R3 	; Subtrahiere Teilerwert
	sbc R2,R4
	inc rmp 	; Ziffer um eins erhhen
	rjmp fpconv10d1 ; und noch einmal von vorne
