; LCD-Basisroutinen fr 4-Bit-Anschlu einer zweizeiligen LCD an einen ATMEL- Port
;
; Problem mit Zeilenumbruch wurde noch nicht gelst
;
; Portx - Bit0..3 -> DB4..DB7 = Datenbits des LCD
; Portx - Bit4 = RS (Register Select)
; Portx - Bit5 = E (Enable Signal)
;
; Definitionen, die in dem aufrufenden Assembler - Programm enthalten sein mssen:
; - Stackoperationen
; - Registerfestlegung / Speicherplatz fr rmp
; - Register R0 wird als Zhler benutzt
; - Z-Register mu temporr zur Verfgung stehen
; - Taktfrequenz ftakt
; - pLcdPort Aktiver LCD-Port
; - pLcdDdr Datenrichtungsregister des aktiven Port
;
; Subroutinen:
; - Lcd4Init: Setzt die LCD zurck
; - Lcd4Chr: Gibt den Character in rmp aus
; - Lcd4PBcd: Gibt die gepackte BCD-Zahl in rmp aus
; - Lcd4ZTxt: Gib nullterminierten Text aus Flash ab Z
; - Lcd4RTxt: Gib rmp chars aus SRAM ab Z aus
;
; Festlegungen von Konstanten fr den LCD-Port
.EQU PowerUpWait=0b00000011	; Initialisierungswort fr LCD
.EQU cLcdWrite=0b11111111 	; Datenrichtung Schreibe LCD
.EQU mLcdRs=0b00010000 		; RS-Bit Maske
.EQU bLcdEn=5 				; Enable -> Bit5
.EQU c1s=200 				; Warten zu Beginn (200 * 5 ms)
.EQU c5ms=ftakt/800 		; 5 ms Warten nach jedem Kontrollwort
.EQU c50us=ftakt/80000 		; 50 us Warten nach jedem Zeichen
;
; Makro fr Enable active time
;
; Version fr 10 MHz Takt
;.MACRO enactive
;	nop
;	nop
;	nop
;	nop
;	nop
;.ENDMACRO
;
;Version fr 8 MHz Takt
.MACRO enactive
	nop
	nop
	nop
	nop
.ENDMACRO
;
; Version fr 6 MHz Takt
;.MACRO enactive
;	nop
;	nop
;	nop
;.ENDMACRO
;
; Version fr 4 MHz Takt
;.MACRO enactive
;	nop
;	nop
;.ENDMACRO
;
; Version fr 2 MHz Takt
;.MACRO enactive
;	nop
;.ENDMACRO
;
Lcd4Init:
	rcall LcdDelay1s 	; Warte eine Sekunde auf LCD
	ldi rmp,cLcdWrite 	; Datenrichtung der Atmel-Ports fr LCD auf Ausgang
	out pLcdDdr,rmp
	ldi rmp,PowerUpWait	; Initialisierungswort zum Abfangen der LCD
	rcall Lcd4Set 		; drei Mal senden mit Delay je 5 ms
	rcall LcdDelay5ms
	ldi rmp,PowerUpWait
	rcall Lcd4Set
	rcall LcdDelay5ms
	ldi rmp,PowerUpWait
	rcall Lcd4Set
	rcall LcdDelay5ms
	ldi rmp,0b00101000 	; Funktionseinstellungen: 	- 4 Bit- Modus
						;							- Mehrzeilenmodus
						;							- 5x7 Punktmatrix pro Zeichen
	rcall Lcd4Ctrl 		; Ausgabe auf Control Port LCD
	rcall LcdDelay5ms
	ldi rmp,0b00101000 	; Funktionseinstellungen (zur Sicherheit ein zweites Mal senden):
						;						 	- 4 Bit- Modus
						;							- Mehrzeilenmodus
						;							- 5x7 Punktmatrix pro Zeichen
	rcall Lcd4Ctrl 		; Ausgabe auf Control Port LCD
	rcall LcdDelay5ms
	ldi rmp,0b00011100 	; Cursoreinstellungen: Scroll display / nach rechts
	rcall Lcd4Ctrl
	ldi rmp,0b00001100 	; LCD an / Curser aus / blinkender Buchstabe auf Curserposition aus
	rcall Lcd4Ctrl
	ldi rmp,0b00000110 	; Entry mode
	rcall Lcd4Ctrl
Lcd4Clear:
	ldi rmp,0b00000001 	; Set Lcd Clear
	rcall Lcd4Ctrl
Lcd4Home:
	ldi rmp,0b00000010 	; Set LCD Home Position
	ret					; Fertig
;
; Ausgabe von rmp an den Control-Port der LCD
;
; Da das obere Nibble (Bit4-Bit7) als erstes bertragen wird, die 4 Datenleitungen
; jedoch an die vier unteren Bits des Portx angeschlossen sind, muss man die beiden
; Nibbles des zu bertragenden Bytes erstmal vertauschen.
;
Lcd4Ctrl:
	push rmp 			; Rette Byte auf Stapel
	swap rmp 			; Vertausche Nibble
	andi rmp,0x0F 		; Lsche unteres Nibble
	rcall Lcd4Set 		; Gib oberes Nibble aus
	pop rmp 			; Stelle Byte wieder her
	andi rmp,0x0F 		; Lsche oberes Nibble
	rcall Lcd4Set 		; Gib unteres Nibble aus
	rjmp LcdDelay5ms 	; 5ms Pause und Fertig
;
; Gib die gepackte BCD-Zahl in rmp auf dem LCD aus
;
Lcd4PBcd:
	push rmp 			; Save on stack
	swap rmp 			; Higher to lower nibble
	rcall Lcd4PBcd1 	; Output nibble
	pop rmp 			; Restore from stack
Lcd4PBcd1:
	andi rmp,0x0F 		; Mask upper nibble
	ori rmp,0x30 		; Nibble to ASCII
	; luft in LCD4Chr hinein
;
; Gibt char (Zeichen als Hex-Zahl) in rmp auf LCD aus -> wie Lcd4Ctrl, aber mit RS-Bit setzen
;
Lcd4Chr:
	push rmp 			; Rette Char auf Stapel
	swap rmp 			; Vertausche Nibble
	andi rmp,0x0F 		; Lsche unteres Nibble
	sbr rmp,mLcdRs 		; Setze RS-Bit in rmp
	rcall Lcd4Set 		; Gibt oberes Nibble aus
	pop rmp 			; Hole Char vom Stapel
	andi rmp,0x0F 		; Lsche oberes Nibble
	sbr rmp,mLcdRs 		; Setze RS-Bit
	rcall Lcd4Set 		; Gibt unteres Nibble aus
	rjmp LcdDelay50us 	; 50s Pause und Fertig
;
; Gib Nibble in rmp an LCD aus
;
Lcd4Set:
	out pLcdPort,rmp 	; Byte auf Ausgabeport
	nop
	sbi pLcdPort,bLcdEn ; Setze Enable-Bit
	enactive 			; Delay macro
	cbi pLcdPort,bLcdEn ; Enable Bit lschen
	nop
	ret					; Ende der Funktion und Rcksprung
;
; Verzgerung um 1 Sekunde bei Init der LCD
;
LcdDelay1s:
	ldi rmp,c1s 		; 200 * 5 ms warten
LcdDelay1s1:
	rcall LcdDelay5ms
	dec rmp
	brne LcdDelay1s1
	ret					; Ende der Funktion und Rcksprung
;
; Verzgerung um 5 ms nach jedem Control Word
;
LcdDelay5ms:
	push ZH
	push ZL
	ldi ZH,HIGH(c5ms)
	ldi ZL,LOW(c5ms)
LcdDelay5ms1:
	sbiw ZL,1
	brne LcdDelay5ms1
	pop ZL
	pop ZH
	ret					; Ende der Funktion und Rcksprung
;
; Delay um 50 Mikrosekunden nach jedem Char
;
LcdDelay50us:
	ldi rmp,c50us
LcdDelay50us1:
	nop
	dec rmp
	brne LcdDelay50us1
	ret					; Ende der Funktion und Rcksprung
;
; Gib an der Position in rmp den Text ab Z aus (null-term.)
;
Lcd4ZTxt:
	sbr rmp,0b10000000 	; Setze DD-RAM-Adresse
	rcall Lcd4Ctrl
Lcd4ZTxt1:
	lpm 				; Get a char
	tst R0 				; Null-Char?
	breq Lcd4ZTxtR
	mov rmp,R0
	rcall Lcd4Chr
	adiw ZL,1
	rjmp Lcd4ZTxt1
Lcd4ZTxtR:
	ret					; Ende der Funktion und Rcksprung

; Gibt eine Anzahl chars Text (bestimmt rmp) im SRAM ab Z aus
Lcd4RTxt:
	mov R0,rmp 			; R0 ist Zhler
Lcd4RTxt1:
	ld rmp,Z+ 			; Lese char
	rcall Lcd4Chr
	dec R0
	brne Lcd4RTxt1
	ret					; Ende der Funktion und Rcksprung

; Sendet die durch die Adresse (16bit) von Z des EEPROM adressierte Zeichenkette (null-term.)
; an das LCD
LcdEepPrint:
		sbic EECR, EEWE			
		rjmp LcdEepPrint		; Warteschleife, bis EEPROM bereit zum Lesen
        out EEARH, ZH
		out EEARL, ZL			; EEPROM-Adresse
		sbi EECR, EERE          ; Lesevorgang starten
        in rmp, EEDR            ; gelesenes Byte nach "rmp"
        tst rmp                 ; auf 0 (Stringende testen)
        breq LcdEepPrintEnd      ; falls 0, Funktion beenden
        rcall Lcd4Chr           ; Zeichenkette an LCD senden
		adiw ZL, 1				; gesamten Zeiger Z um 1 erhhen
        rjmp LcdEepPrint        ; ... und zum Anfang der Funktion
LcdEepPrintEnd:
        ret						; Ende der Funktion und Rcksprung
